// Receiver module using nRF24L01+ for HiSKY H-6 Transmitter
// 2017-01-27  T. Nakagawa

#include "config.h"
#include "Arduino.h"
#include "HiskyRX.h"
#include "NRF24L01.h"

#ifdef HISKY_RX

#define BIND_CHANNEL 81
#define BIND_ADDRESS ((const uint8_t *)"\x12\x23\x23\x45\x78")
#define PACKET_SIZE 10
#define TIMEOUT 10

uint16_t hisky_rx_values[8];
uint8_t hisky_rx_data_address[5];
uint8_t hisky_rx_data_channels[20];
static NRF24L01 radio(NRF24L01_PIN_CE, NRF24L01_PIN_CS);
static uint8_t state = 0;  // 0-3: Binding, 4: Bound, 5: Data receiving.

void initHiskyRX() {
  radio.setCRC(NRF24L01::CRC_2BYTE);
  radio.setDataRate(NRF24L01::DR_1MBPS);
  radio.setPipes(1);
  radio.setAddressWidth(5);
  radio.setPayloadSize(PACKET_SIZE);
  radio.setAddress(0, BIND_ADDRESS);
  radio.setChannel(BIND_CHANNEL);
  radio.enable();
  if (hisky_rx_data_channels[0] != 0) {
    state = 4;
  }
}

HiskyRxStatus readHiskyRX() {
  static union {uint16_t uint16; uint8_t uint8[2];} adrs_sum = {0};
  static uint8_t hop = 0;
  HiskyRxStatus status = HISKY_RX_IDLE;

  if (state == 4) {
    radio.disable();
    radio.setAddress(0, hisky_rx_data_address);
    radio.setChannel(hisky_rx_data_channels[hop++]);
    radio.enable();
    radio.flush();
    state++;
  }

  while (!radio.empty()) {
    uint8_t buf[PACKET_SIZE];
    radio.read(buf);

    if (state == 0) {
      if (buf[0] == 0xff && buf[1] == 0xaa && buf[2] == 0x55) {
        for (uint8_t i = 0; i < 5; i++) {
          adrs_sum.uint16 += (hisky_rx_data_address[i] = buf[3 + i]);
        }
        state++;
      }
    } else if (state < 4) {
      if (buf[0] == adrs_sum.uint8[0] && buf[1] == adrs_sum.uint8[1] && buf[2] == state - 1) {
        for (uint8_t i = 0; i < 7; i++) {
          hisky_rx_data_channels[7 * (state - 1) + i] = buf[3 + i];
          if (state == 3 && i == 5) {
            state++;
            return HISKY_RX_BOUND;
          }
        }
        state++;
      }
    } else {
      radio.disable();
      radio.setChannel(hisky_rx_data_channels[hop++]);
      if (hop >= 20) hop = 0;
      radio.enable();

      uint16_t high = *(uint16_t *)(buf + 8);
      for (int8_t i = 0; i < 8; i++) {
        uint16_t val = (((high & 0x0003) << 8) | buf[i]);
        high >>= 2;
        if (i == 6 && val == 0x03) val = 0x3ff;
        hisky_rx_values[i] = val;
      }
      status = HISKY_RX_UPDATED;
    }
  }

  return status;
}

#endif

